// Creative Commons Attribution-Share Alike 3.0 Unported Licence
// http://creativecommons.org/licenses/by-sa/3.0/

var CGD = window.CGD || {};
CGD.ASTRO = CGD.ASTRO || {};

// At this point the only calculations here are for daylight.  I got the method
// from http://www.srrb.noaa.gov/highlights/sunrise/calcdetails.html
// especially: http://www.srrb.noaa.gov/highlights/sunrise/solareqns.PDF

// r_ radians
// d_ degrees
// m_ minutes

// dt_ date, function returning
// ms_ milliseconds, function returning

// The current functions depend on both location on the earth,
//  and the on the current time.
// pacha is a function taking a place, and it can also take a function
//  which returns the current time ({dt_now: function() {...}})
//  (dependency injection)
//  The result is a pacha: a moment of space-time.
// Pacha is a Quechua (Incan) word meaning roughly "time ground place"
//  (Actually not quite right since time here a function, and a pacha is
//  a certain place at a certain time.)
//  Source: William Sullivan, "The Secret of the Incas", 
//    New York: Three Rivers Press, 1996
CGD.ASTRO.pacha = function(d_latitude, d_longitude, options) {
  options = options || {};

  options.dt_now = options.dt_now || CGD.ASTRO.defaultNow;

  // Winter doesn't have daylight savings
  // We can't assume winter in december because of the southern hemisphere
  // I don't know if anybody down there uses daylight crazy time, 
  //   we can't trust it for hemisphere detection, but we might as well use it
  //   if the evidence is there.
  // This used to operate with ms_now, but we need the timezone information,
  //   we could re-create a date object, but ms_now incorporated the offset.
  (function() {
    // The new Date here makes sure we have an independent instance,
    //   since we have to modify it.
    var dt = new Date(options.dt_now());
    
    dt.setMonth(12-1);
    var december = dt.getTimezoneOffset()/-4;
    
    dt.setMonth(6-1);
    var june = dt.getTimezoneOffset()/-4;
    
    if (december > june) {
      d_latitude = d_latitude || -45;
      d_longitude = d_longitude || june;
    } else {
      d_latitude = d_latitude || 45;
      d_longitude = d_longitude || december;
    }
  }());
  
  var RADIANS = 2 * Math.PI;
  
  function radiansFromDegrees(degrees) {
    return degrees * RADIANS / 360;
  }

  function degreesFromRadians(radians) {
    return radians * 360 / RADIANS;
  }

  var MS_YEAR = 1000 * 60 * 60 * 24 * 365.24219;
  function r_gamma() {
    return ((options.dt_now().getTime() / MS_YEAR) % 1) * RADIANS;
  }

  // equation of time
  function m_eqtime() {
    var y = r_gamma();
    return 229.18 * (
      0.0000075
      + 0.001868 * Math.cos(y)
      - 0.032077 * Math.sin(y)
      - 0.014615 * Math.cos(y * 2)
      - 0.040849 * Math.sin(y * 2)
    );
  }

  function r_declination() {
    var y = r_gamma();
    return 0.006918
      - 0.399912 * Math.cos(y)
      + 0.070257 * Math.sin(y)
      - 0.006758 * Math.cos(y * 2)
      + 0.000907 * Math.sin(y * 2)
      - 0.002697 * Math.cos(y * 3)
      + 0.001480 * Math.sin(y * 3);
  }

  function m_hourAngle(d_zenith) {
    var lat = radiansFromDegrees(d_latitude);
    var decl = r_declination();
    var c_zenith = Math.cos(radiansFromDegrees(d_zenith));
    var divisor = Math.cos(lat)*Math.cos(decl);
    var tanget = Math.tan(lat)*Math.tan(decl);
    var first = c_zenith / divisor;
    var delta = Math.min(first - tanget, 1);
    var hourAngle = 4 * degreesFromRadians(Math.acos(delta));
    return hourAngle;
  }

  function m_solarNoon() {
    return (12 * 60) - 4*d_longitude - options.dt_now().getTimezoneOffset() - m_eqtime();
  }
  
  return {m_hourAngle: m_hourAngle, m_solarNoon: m_solarNoon};
};

// These are broken out so that they can be globally replaced if better
// information is available.
CGD.ASTRO.defaultNow = function() {return new Date();};
